#include "FBullCowGame.h"
#include <map>
#define TMap std::map // To make syntax Unreal friendly

using int32 = int; // To make syntax Unreal friendly


// Constructor
FBullCowGame::FBullCowGame() { Reset(); }

// GETters
int32 FBullCowGame::GetCurrentTry()			const { return MyCurrentTry; }
int32 FBullCowGame::GetHiddenWordLength()	const { return MyHiddenWord.length(); } // This is typecasting to int32
bool  FBullCowGame::IsGameWon()				const { return bGameIsWon; }

// Used for difficulty flow
int32 FBullCowGame::GetMaxTries() const
{
	TMap<int32, int32> WordLengthToMaxTries{ {3,4}, {4,7}, {5,10}, {6,16}, {7,20} };
	return WordLengthToMaxTries[MyHiddenWord.length()];
}

// sets hidden word, current try
void FBullCowGame::Reset()
{
	const FString HIDDEN_WORD = "and"; // this MUST be an isogram!

	bGameIsWon = false;
	MyHiddenWord = HIDDEN_WORD;

	MyCurrentTry = 1;
	return;
}


EGuessStatus FBullCowGame::CheckGuessValidity(FString Guess) const
{
	if (!IsIsogram(Guess))								{ return EGuessStatus::Not_Isogram; }
	else if (!IsLowerCase(Guess))						{ return EGuessStatus::Not_English_Lower_Case_Letters; }
	else if (Guess.length() != GetHiddenWordLength())	{ return EGuessStatus::Wrong_Length; }
	else												{ return EGuessStatus::OK;	}
}

// receives a VALID guess, increments turn and returns count
FBullCowCount FBullCowGame::SubmitValidGuess(FString Guess)
{
	MyCurrentTry++;
	FBullCowCount BullCowCount;
	int32 WordLength = MyHiddenWord.length();	// assuming same length as guess

	// loop through all letters in the hidden  word
	for (int32 MHWChar = 0; MHWChar < WordLength; MHWChar++)
	{
		// compare letters against guess
		for (int32 GChar = 0; GChar < WordLength; GChar++)
		{
			if (Guess[GChar] == MyHiddenWord[MHWChar])
			{
				if (GChar == MHWChar)
					BullCowCount.Bulls++;
				else
					BullCowCount.Cows++;
			}
		}
	}

	if (BullCowCount.Bulls == WordLength)
	{
		bGameIsWon = true;
	}

	return BullCowCount;
}


bool FBullCowGame::IsIsogram(FString Word) const
{
	TMap<char, bool> LetterSeen; // setup our map

	for(auto Letter : Word)
	{
		Letter = tolower(Letter); //  handle mixed case

		if (LetterSeen[Letter]) { return false; }

		LetterSeen[Letter] = true;
	}
	return true; // for example if /0 is entered
}

// checks if each letter in isogram is a lower case english letter
bool FBullCowGame::IsLowerCase(FString Word) const
{
	for (auto Letter : Word)
	{
		if (!islower(Letter))
			return false;
	}
	return true;
}
