/*
 * FBullCowGame class
 * Creates a BullCowGame object
 * made by Nathanael Whitrow aka eleCt_eXile
 * For bugs and improvements email me at nathanael.whitrow@gmail.com
 * The game logic (no view code or direct user interaction)
 * The game is a simple guess the word game based on master-mind
*/


#pragma once
#include <string>

// To make syntax Unreal friendly
using FString = std::string;
using int32 = int;

// All values initialised to 0
struct FBullCowCount
{
	int32 Bulls = 0;
	int32 Cows = 0;
};

enum class EGuessStatus
{
	Invalid_Status,
	OK,
	Not_Isogram,
	Wrong_Length,
	Not_English_Lower_Case_Letters
};


// Define the interface
class FBullCowGame
{
public:
	
	// Constructor
	FBullCowGame();


	int32 GetMaxTries()				const;
	int32 GetCurrentTry()			const;
	int32 GetHiddenWordLength()		const;
	bool IsGameWon()				const;

	EGuessStatus CheckGuessValidity(FString)	const;
	
	void Reset();
	FBullCowCount SubmitValidGuess(FString);

private:
	// See constructor for initialisation
	int32 MyCurrentTry;
	FString MyHiddenWord;
	bool bGameIsWon;
	
	// Helper fncs
	bool IsIsogram(FString)			const;
	bool IsLowerCase(FString)		const;
};